#include "stdafx.h"
#include "ball.h"

#include "model.h"

TRAVERSABLE(brixx::Ball)

namespace brixx
{

#define MIN_LAUNCH_SPEED_X -32.f
#define MAX_LAUNCH_SPEED_X 32.f
#define LAUNCH_SPEED_Y -320.f
#define PADDLE_MULTIPLIER -1.1f
#define PADDLE_SPEED 768.f
#define BRICK_MULTIPLIER 0.75f

void Ball::resetAll(const Model& aModel)
{
	Ball* b(first());
	while(b)
	{
		b->reset(aModel);
		b = b->next();
	}
}

void Ball::updateAll(const Model& aModel, IEventTarget& aTarget)
{
	Ball* b(first());
	while(b)
	{
		b->update(aModel, aTarget);
		b = b->next();
	}
}

Ball::Ball(const Vector2f& aPosition)
:myPosition(aPosition)
,mySpeed(0.f, 0.f)
{
}

void Ball::update(const Model& aModel, IEventTarget& aTarget)
{
	mySpeed.y += Model::GRAVITY * Model::SECONDS_PER_TICK;
	myPosition += mySpeed * Model::SECONDS_PER_TICK;

	//edges
	if(myPosition.x < 0.f)
	{
		myPosition.x = 0.f;
		mySpeed.x = -mySpeed.x;
		aTarget.edge();
	}
	else if(myPosition.x > (Model::PLAYFIELD_WIDTH - Model::BALL_ASPECT))
	{
		myPosition.x = (Model::PLAYFIELD_WIDTH - Model::BALL_ASPECT) - 1;
		mySpeed.x = -mySpeed.x;
		aTarget.edge();
	}

	if(myPosition.y < 0.f)
	{
		myPosition.y = 0.f;
		mySpeed.y = -mySpeed.y;
		aTarget.edge();
	}
	else if(myPosition.y > (Model::PLAYFIELD_HEIGHT - Model::BALL_ASPECT))
	{
		//miss
		if(myPosition.x < (aModel.myPaddle - Model::BALL_ASPECT) ||
			myPosition.x > (aModel.myPaddle + Model::PADDLE_WIDTH))
		{
			aTarget.miss();
			mySpeed.y = LAUNCH_SPEED_Y;
		}
		//paddle
		else
		{
			myPosition.y = (Model::PLAYFIELD_HEIGHT - Model::BALL_ASPECT);

			mySpeed.y *= PADDLE_MULTIPLIER;

			float f(myPosition.x - aModel.myPaddle);
			f -= (Model::PADDLE_WIDTH - Model::BALL_ASPECT) * 0.5f;
			f /= Model::PADDLE_WIDTH;
			assert(f >= -1.f && f <= 1.f);
			mySpeed.x = PADDLE_SPEED * f;

			aTarget.paddle();
		}
	}

	//brixx
	const Brick* T[4] =
	{
		aModel.test(*this, Vector2f(0.f, 0.f)),
		aModel.test(*this, Vector2f(Model::BALL_ASPECT, 0.f)),
		aModel.test(*this, Vector2f(0.f, Model::BALL_ASPECT)),
		aModel.test(*this, Vector2f(Model::BALL_ASPECT, Model::BALL_ASPECT)),
	};
	if(T[0]->isOn() && T[1]->isOn())
	{
		myPosition.y = (float)((((int)myPosition.y / Model::BRICK_HEIGHT) + 1) * Model::BRICK_HEIGHT);
		mySpeed.x *= BRICK_MULTIPLIER;
		mySpeed.y *= -BRICK_MULTIPLIER;
		aTarget.breakBrick(T[0]);
		if(T[1] != T[0])
			aTarget.breakBrick(T[1]);
		aTarget.brick();
	}
	else if(T[0]->isOn() && T[2]->isOn())
	{
		myPosition.x = (float)((((int)myPosition.x / Model::BRICK_HEIGHT) + 1) * Model::BRICK_HEIGHT);
		mySpeed.x *= -BRICK_MULTIPLIER;
		aTarget.breakBrick(T[0]);
		if(T[2] != T[0])
			aTarget.breakBrick(T[2]);
		aTarget.brick();
	}
	else if(T[1]->isOn() && T[3]->isOn())
	{
		myPosition.x = (float)((((int)myPosition.x / Model::BRICK_HEIGHT) + 1) * Model::BRICK_HEIGHT - Model::BALL_ASPECT);
		mySpeed.x *= -BRICK_MULTIPLIER;
		aTarget.breakBrick(T[1]);
		if(T[3] != T[1])
			aTarget.breakBrick(T[3]);
		aTarget.brick();
	}
	else if(myPosition.y < (Model::PLAYFIELD_HEIGHT - Model::BALL_ASPECT) && T[2]->isOn() && T[3]->isOn())
	{
		myPosition.y = (float)((((int)myPosition.y / Model::BRICK_HEIGHT) + 1) * Model::BRICK_HEIGHT - Model::BALL_ASPECT);
		mySpeed.y = -mySpeed.y;
		aTarget.breakBrick(T[2]);
		if(T[3] != T[2])
			aTarget.breakBrick(T[3]);
		aTarget.brick();
	}
}

void Ball::reset(const Model& aModel)
{
	myPosition.set((float)(aModel.myPaddle + (Model::PADDLE_WIDTH - Model::BALL_ASPECT) / 2), (float)(Model::PLAYFIELD_HEIGHT - Model::BALL_ASPECT));
	mySpeed.set(FRAND(MIN_LAUNCH_SPEED_X, MAX_LAUNCH_SPEED_X), LAUNCH_SPEED_Y);
}


}