#include "stdafx.h"
#include "debris.h"

#include "image.h"
#include "model.h"

TRAVERSABLE(brixx::Debris)

namespace brixx
{

#define SPEED 32.f

void Debris::drawAll(const float aGravity, const Image& anImage, const Vector2i& anOrigin, Canvas& aCanvas)
{
	Debris* d(Debris::first());
	while(d)
	{
		if(d->draw(aGravity, anImage, anOrigin, aCanvas))
		{
			d = d->next();
		}
		else
		{
			Debris* tmp(d->next());
			delete d;
			d = tmp;
		}
	}
}

Debris::Debris(const int aX, const int aY)
:myPosition((float)aX, (float)aY)
,mySpeed(FRAND(-SPEED, SPEED), FRAND(-SPEED, SPEED))
{
}

const bool Debris::draw(const float aGravity, const Image& anImage, const Vector2i& anOrigin, Canvas& aCanvas)
{
	mySpeed.y += aGravity * Model::SECONDS_PER_TICK;
	myPosition += mySpeed * Model::SECONDS_PER_TICK;
	if(myPosition.x < -anImage.width() || myPosition.x > Model::PLAYFIELD_WIDTH ||
		myPosition.y < -anImage.height() || myPosition.y > Model::PLAYFIELD_HEIGHT)
	{
		return false;
	}

	anImage.draw(aCanvas, anOrigin.x + (int)myPosition.x, anOrigin.y + (int)myPosition.y);
	return true;
}

}