#include "stdafx.h"
#include "model.h"

#include "ball.h"

namespace brixx
{

//IEventTarget
void Model::setInput(const int aX)
{
	myPaddle = aX;
	if(myPaddle < 0)
		myPaddle = 0;
	else if(myPaddle >= (PLAYFIELD_WIDTH - PADDLE_WIDTH))
		myPaddle = (PLAYFIELD_WIDTH - PADDLE_WIDTH);
}

void Model::edge()
{
	myTarget.edge();
}

void Model::miss()
{
	myTarget.miss();
}

void Model::paddle()
{
	myTarget.paddle();
}

void Model::brick()
{
	if(allBrixxCleared())
		reset();

	myTarget.brick();
}

void Model::breakBrick(const Brick* aBrick)
{
	unsigned int i;

	for(i = 0; i < BRIXX_X * BRIXX_Y; i++)
	{
		if(aBrick == (myBrixx + i))
		{
			if(myBrixx[i].isOn())
				myBrixx[i].set(false);
			break;
		}
	}

	myTarget.breakBrick(aBrick);
}

void Model::reset()
{
	resetBrixx();
	Ball::resetAll(*this);

	myTarget.reset();
}

void Model::design(const int aX, const int aY, const bool aValue)
{
	designBrick(aX, aY, aValue);

	myTarget.design(aX, aY, aValue);
}

//specific
Model::Model(IEventTarget& aTarget)
:myTarget(aTarget)
,myPaddle(0)
{
	autoDesignBrixx();
	new Ball(Vector2f(0.f, 0.f));
	resetBrixx();
	Ball::resetAll(*this);
}

Model::~Model()
{
	Ball::deleteAll();
}

void Model::update()
{
	Ball::updateAll(*this, *this);}

//private
void Model::autoDesignBrixx()
{
	int x, y;

	for(y = 0; y < Model::BRIXX_Y; y++)
	{
		for(x = 0; x < Model::BRIXX_X; x++)
		{
			designBrick(x, y, y < (BRIXX_Y - 4) && y >= 4);
		}
	}
}

void Model::resetBrixx()
{
	int x, y;

	for(y = 0; y < Model::BRIXX_Y; y++)
	{
		for(x = 0; x < Model::BRIXX_X; x++)
		{
			resetBrick(x, y);
		}
	}
}

const Brick* Model::test(const Ball& aBall, const Vector2f& anOffset) const
{
	const Vector2i BS((int)(aBall.myPosition.x + anOffset.x) / BRICK_WIDTH, (int)(aBall.myPosition.y + anOffset.y) / BRICK_HEIGHT);
	if(validBrick(BS.x, BS.y))
	{
		return myBrixx + BS.x + BS.y * BRIXX_X;
	}
	else
	{
		static Brick null;
		null.set(false);
		return &null;
	}
}

void Model::designBrick(const int aX, const int aY, const bool aValue)
{
	if(validBrick(aX, aY))
		myBrixx[aX + aY * BRIXX_X].design(aValue);
}

void Model::resetBrick(const int aX, const int aY)
{
	if(validBrick(aX, aY))
		myBrixx[aX + aY * BRIXX_X].reset();
}

const bool Model::brick(const int aX, const int aY) const
{
	if(validBrick(aX, aY))
		return myBrixx[aX + aY * BRIXX_X].isOn();
	else
		return true;
}

const bool Model::validBrick(const int aX, const int aY) const
{
	return aX >= 0 && aX < BRIXX_X && aY >= 0 && aY < BRIXX_Y;
}

const bool Model::allBrixxCleared() const
{
	int x, y;

	for(y = 0; y < Model::BRIXX_Y; y++)
	{
		for(x = 0; x < Model::BRIXX_X; x++)
		{
			if(brick(x, y))
				return false;
		}
	}

	return true;
}

const Vector2i Model::brickSpace(const Brick* aBrick) const
{
	unsigned int i;

	for(i = 0; i < BRIXX_X * BRIXX_Y; i++)
	{
		if(aBrick == (myBrixx + i))
		{
			return Vector2i(i % BRIXX_X, i / BRIXX_X);
		}
	}

	return Vector2i(0, 0);
}

const float Model::GRAVITY(512.f), Model::SECONDS_PER_TICK(1.f / 50.f);
const unsigned int Model::MILLISECONDS_PER_TICK(1000 / 50);

}