#pragma once

#include "ieventtarget.h"
#include "brick.h"
#include "vector.h"

namespace brixx
{

class Ball;

class Model : public IEventTarget
{
public:

	//IEventTarget
	void setInput(const int aX);
	void edge();
	void miss();
	void paddle();
	void brick();
	void breakBrick(const Brick* aBrick);
	void reset();
	void design(const int aX, const int aY, const bool aValue);

	//specific
	Model(IEventTarget& aTarget);
	~Model();
	void update();
	const bool brick(const int aX, const int aY) const;
	const Brick* test(const Ball& aBall, const Vector2f& anOffset) const;
	const Vector2i brickSpace(const Brick* aBrick) const;

	enum
	{
		PADDLE_WIDTH = 66,
		PADDLE_HEIGHT = 16,
		BRICK_WIDTH = 32,
		BRICK_HEIGHT = 32,
		BALL_ASPECT = 16,
		BRIXX_X = 16,
		BRIXX_Y = 24,
		PLAYFIELD_WIDTH = BRICK_WIDTH * BRIXX_X,
		PLAYFIELD_HEIGHT = BRICK_HEIGHT * BRIXX_Y,
	};

	//members
	static const float GRAVITY, SECONDS_PER_TICK;
	static const unsigned int MILLISECONDS_PER_TICK;
	int myPaddle;
	
private:

	const bool allBrixxCleared() const;
	void autoDesignBrixx();
	void resetBrixx();
	void designBrick(const int aX, const int aY, const bool aValue);
	void resetBrick(const int aX, const int aY);
	const bool validBrick(const int aX, const int aY) const;
	
	//members
	IEventTarget& myTarget;
	Brick myBrixx[BRIXX_X * BRIXX_Y];
};

}