#include "stdafx.h"
#include "playcontroller.h"

#include "canvas.h"
#include "assets.h"
#include "model.h"
#include "ball.h"
#include "debris.h"

namespace brixx
{

#define ASSET_EDGE L"edge.wav"
#define ASSET_MISS L"miss.wav"
#define ASSET_PADDLE L"paddle.wav"
#define ASSET_BRICK L"brick.wav"

//IController
const bool PlayController::draw(Canvas& aCanvas)
{
	myTarget.setInput(aCanvas.cursorPos().x);

	drawBrixx(aCanvas);

	Debris::drawAll(Model::GRAVITY, myAssets.myDebris, ORIGIN, aCanvas);

	myAssets.myPaddle.draw(aCanvas, ORIGIN.x + myModel.myPaddle, ORIGIN.y + Model::PLAYFIELD_HEIGHT);

	const Ball* b(Ball::first());
	while(b)
	{
		myAssets.myBall.draw(aCanvas, ORIGIN.x + (int)b->myPosition.x, ORIGIN.y + (int)b->myPosition.y);
		b = b->next();
	}

	return true;
}

//IEventTarget
void PlayController::edge()
{
	::PlaySound(ASSET_EDGE, NULL, SND_FILENAME | SND_ASYNC);
}

void PlayController::miss()
{
	::PlaySound(ASSET_MISS, NULL, SND_FILENAME | SND_ASYNC);
}

void PlayController::paddle()
{
	::PlaySound(ASSET_PADDLE, NULL, SND_FILENAME | SND_ASYNC);
}

void PlayController::brick()
{
	::PlaySound(ASSET_BRICK, NULL, SND_FILENAME | SND_ASYNC);
}

void PlayController::breakBrick(const Brick* aBrick)
{
	const Vector2i BS(myModel.brickSpace(aBrick));
	int x, y;

	for(y = 0; y < Model::BRICK_HEIGHT / myAssets.myDebris.height(); y++)
	{
		for(x = 0; x < Model::BRICK_WIDTH / myAssets.myDebris.width(); x++)
		{
			new Debris(BS.x * Model::BRICK_WIDTH + x * myAssets.myDebris.width(), BS.y * Model::BRICK_HEIGHT + y * myAssets.myDebris.height());
		}
	}
}

void PlayController::reset()
{
	Debris::deleteAll();
}

//specific
PlayController::PlayController(const Assets& someAssets, const Model& aModel, IEventTarget& aTarget)
:IController(someAssets, aModel, aTarget)
{
}

PlayController::~PlayController()
{
	Debris::deleteAll();
}

}