#include "stdafx.h"
#include "time.h"

#include "tgttimer.h"

namespace brixx
{

static TGTTimer locTimer;
static float locNow(locTimer.now());
static float locElapsed(0.f);

void time::update()
{
	const float OLDTIME(locNow);
	locNow = locTimer.now();
	locElapsed = locNow - OLDTIME;
}

void time::externalUpdate(const float aCurrentTime, const float anElapsedTime)
{
	locNow = aCurrentTime;
	locElapsed = anElapsedTime;
}

const unsigned int time::frame(const float anFps, const unsigned int aNumFrames)
{
	if(anFps == 0.f)
		return 0;
	else
		return (unsigned int)(locNow * anFps) % aNumFrames;
}

const float time::floatFrame(const float aNow, const float anFps, const unsigned int aNumFrames)
{
	if(anFps == 0.f)
		return 0.f;
	
	float frame(aNow * anFps);
	while(frame > aNumFrames)
		frame -= aNumFrames;

	return frame;
}

const unsigned int time::frameFromTime(const float anFps, const unsigned int aNumFrames, const float aStartTime)
{
	if(anFps == 0.f)
		return 0;
	else
		return (unsigned int)((locNow - aStartTime) * anFps) % aNumFrames;
}

const float time::fps()
{
	return 1.f / locElapsed;
}

const float time::now()
{
	return locNow;
}

const float time::future(const float aTimeFromNow)
{
	return now() + aTimeFromNow;
}

const float time::elapsed()
{
	return locElapsed;
}

const bool time::isFuture(const float aTime)
{
	return !isPast(aTime);
}

const bool time::isPast(const float aTime)
{
	return aTime <= now();
}

}