#pragma once

namespace brixx
{

#define TRAVERSABLE(x) x* brixx::Traversable<x>::ourFirstInstance(NULL); x* brixx::Traversable<x>::ourLastInstance(NULL); unsigned int brixx::Traversable<x>::ourNumInstances(0);

template <class X>
class Traversable
{
public:

	static X* first()
	{
		return ourFirstInstance;
	}

	static X* last()
	{
		X* i(ourFirstInstance);
		while(i && i->myNextInstance)
			i = i->myNextInstance;
		return i;
	}

	static void deleteAll()
	{
		while(ourFirstInstance)
			delete ourFirstInstance;
	}

	virtual ~Traversable()
	{
		//unlink
		if(this == ourFirstInstance)
		{
			ourFirstInstance = myNextInstance;

			if(this == ourLastInstance)
			{
				assert(!ourFirstInstance);
				ourLastInstance = NULL;
			}
		}
		else
		{
			X* previous = ourFirstInstance;
			X* current = previous->Traversable<X>::myNextInstance;

			while(current != this)
			{
				previous = current;
				current = current->Traversable<X>::myNextInstance;
			}

			previous->Traversable<X>::myNextInstance = Traversable<X>::myNextInstance;

			if(this == ourLastInstance)
				ourLastInstance = previous;
		}

		//decrease instance count
		assert(ourNumInstances);
		ourNumInstances--;
	}

	X* next() const
	{
		return myNextInstance;
	}

	X* nextWrap() const
	{
		if(myNextInstance)
			return myNextInstance;
		else
			return Traversable<X>::first();
	}

	X* previousWrap() const
	{
		X* i(first());

		//nothing in list
		if(!i)
			return NULL;

		//this is the first instance, so return the last
		if(i == this)
		{
			while(i->myNextInstance)
				i = i->next();
			return i;
		}

		//search for the one before this
		while(i->myNextInstance != this)
			i = i->next();

		return i;
	}

	static const unsigned int count()
	{
		return ourNumInstances;
	}

	static X* find(const unsigned int aKey)
	{
		X* current = first();

		while(current)
		{
			if(current->Traversable<X>::myKey == aKey)
				return current;
			else
				current = current->Traversable<X>::next();
		}

		return NULL;
	}

	static X* find(const X* anInstance)
	{
		X* current = first();

		while(current)
		{
			if(current == anInstance)
				return current;
			else
				current = current->Traversable<X>::next();
		}

		return NULL;
	}

protected:

	Traversable()
	:myNextInstance(NULL)
	{
		//link
		if(ourLastInstance)
		{
			assert(ourFirstInstance);
			ourLastInstance->Traversable<X>::myNextInstance = (X*)this;
			ourLastInstance = (X*)this;
		}
		else
		{
			assert(!ourFirstInstance);
			ourFirstInstance = ourLastInstance = (X*)this;
		}

		//increase instance count
		ourNumInstances++;
	}

private:

	//members
	static X* ourFirstInstance;
	static X* ourLastInstance;
	static unsigned int ourNumInstances;
	X* myNextInstance;
};

}
