#pragma once

namespace brixx
{

//FORWARD DECLARATIONS
template <class Type> class Vector3;
template <class Type> class Vector4;


//2D VECTOR
template <class Type>
class Vector2
{
public:

	//CONSTRUCTORS
	Vector2()															{}
    Vector2(Type aX, Type aY)											{x = aX; y = aY;}
	
	template <class AnotherType> 
	Vector2(const Vector2<AnotherType>& aVector)					{x = (Type)aVector.x; y = (Type)aVector.y;}

	//DESTRUCTOR
	~Vector2()															{}

	//ASSIGNMENT
	void set(Type aX, Type aY)												{x = aX; y = aY;}
	template <class AnotherType> 
	Vector2<Type>& operator=(const Vector2<AnotherType>& aVector)	{x = (Type)aVector.x; y = (Type)aVector.y; return *this;}
	Vector2<Type>& operator=(const Vector2<Type>& aVector)			{x = aVector.x; y = aVector.y; return *this;}

    //MODIFYING OPERATIONS
    Vector2<Type>& operator+=(const Vector2<Type>& aVector)		{x += aVector.x; y += aVector.y; return *this;}
    Vector2<Type>& operator-=(const Vector2<Type>& aVector)		{x -= aVector.x; y -= aVector.y; return *this;}
    
	Vector2<Type>& operator*=(Type aValue)								{x *= aValue; y *= aValue; return *this;}
    Vector2<Type>& operator/=(Type aValue)								{x /= aValue; y /= aValue; return *this;}

    //CREATIVE OPERATORS WITH VECTORS
    Vector2<Type> operator+(const Vector2<Type>& aVector) const	{return Vector2<Type>(x+aVector.x, y+aVector.y);}
    Vector2<Type> operator-(const Vector2<Type>& aVector) const	{return Vector2<Type>(x-aVector.x, y-aVector.y);}

    //CREATIVE OPERATORS WITH SCALARS
    Vector2<Type> operator*(Type aValue) const							{return Vector2<Type>(x * aValue, y * aValue);}
    Vector2<Type> operator/(Type aValue) const							{return Vector2<Type>(x / aValue, y / aValue);}

    //COMPARISON
	const bool operator==(const Vector2<Type>& aVector) const				{return (x == aVector.x && y == aVector.y);} 
    const bool operator!=(const Vector2<Type>& aVector) const				{return !(*this == aVector);} 
	
	//NEGATION
	Vector2<Type> operator-() const										{return Vector2<Type>(-x, -y);}

	//MATH
	Vector2<Type> normal() const											{return Vector2<Type>(y, -x);}
	Type length() const														{return (Type)(::sqrtf((Type)(x * x + y * y)));}
    Type length2() const													{return (x * x + y * y);}
    Vector2<Type>& scale(Type aVector)								{normalize(); operator *= (aVector); return *this;}
    Vector2<Type>& normalize()											{operator /= (length()); return *this;}
    
	//KEEP (FLOAT SPECIFIC) ?
	Vector2<Type>& normalizeSafe()										{if(length2() > (Type)0.00001)  normalize() ; else {x = 0; y = 1;} return *this;}
	//KEEP (FLOAT SPECIFIC) ?

	Vector2<Type> unit() const											{return Vector2<Type>(*this).normalize();}

	//KEEP (FLOAT SPECIFIC) ?
	Vector2<Type> unitSafe() const										{return Vector2<Type>(*this).normalizeSafe();}
	//KEEP (FLOAT SPECIFIC) ?

	
	//UTILITY
	//true if the vector is right of aVect
	bool rightOf(const Vector2<Type>& aVect) const						{return ((x * aVect.y) < (y * aVect.x));}

	//true if the vector is left of aVect
	bool leftOf(const Vector2<Type>& aVect) const						{return ((x * aVect.y) > (y * aVect.x));}

	//if deviation > 0, the vector is right of aVect
	//if deviation < 0, the vector is left of aVect
	Type deviation(const Vector2<Type>& aVect) const						{return ((x * aVect.y) - (y * aVect.x));}

	//create aVector vector rotated 90 degrees right
	Vector2<Type> createRightNormal() const								{return Vector2<Type>(-y, x);}

	//create aVector vector rotated 90 degrees left
	Vector2<Type> createLeftNormal()	const								{return Vector2<Type>(y, -x);}

	//Rotate the vector
	Vector2<Type>& rotate(float anAngle)			
	{ 
		if(anAngle != 0.f)
		{
			double sinVal = ::sinf(anAngle);
			double cosVal = ::cosf(anAngle);
			Type temp;

			temp = (Type) (x * cosVal + y * sinVal);
			y = (Type) (y * cosVal - x * sinVal);
			x = temp;
		}

		return *this;
	}

	//dot product
	Type dot(const Vector2<Type>& aVector) const						{return aVector.x*x + aVector.y*y;}

	//members
	Type x, y;
};


//3D VECTOR
template <class Type>
class Vector3
{
public:

	//CONSTRUCTORS
	Vector3()															{}
    Vector3(Type aX, Type aY, Type aZ)									{x = aX; y = aY; z = aZ;}

//	template <class AnotherType> 
//	Vector3(const Vector2<AnotherType>& aVector)					{x = (Type)aVector.x; y = 0; z = (Type)aVector.y;}

	template <class AnotherType> 
	Vector3(const Vector3<AnotherType>& aVector)					{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z;}

//	template <class AnotherType> 
//	Vector3(const Vector4<AnotherType>& aVector)					{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z;}

	//DESTRUCTOR
	~Vector3()															{}
    
	//ASSIGNMENT
	void set(Type aX, Type aY, Type aZ)										{x = aX; y = aY; z = aZ;}

	template <class AnotherType> 
	Vector3<Type>& operator=(const Vector3<AnotherType>& aVector) 	{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; return *this;}

//	template <class AnotherType> 
//	Vector3<Type>& operator=(const Vector4<AnotherType>& aVector) 	{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; return *this;}

	//MODIFYING OPERATIONS
    Vector3<Type>& operator+=(const Vector3<Type>& aVector)		{x += aVector.x; y += aVector.y; z+= aVector.z; return *this;}
    Vector3<Type>& operator-=(const Vector3<Type>& aVector)		{x -= aVector.x; y -= aVector.y; z-= aVector.z; return *this;}
    
//    Vector3<Type>& operator*=(const Vector3<Type>& aVector)		{x *= aVector.x; y *= aVector.y; z *= aVector.z; return *this;}

    Vector3<Type>& operator*=(Type aValue)								{x *= aValue; y *= aValue; z *= aValue; return *this;}
    Vector3<Type>& operator/=(Type aValue)								{x /= aValue; y /= aValue; z /= aValue; return *this;}

    //CREATIVE OPERATORS
    Vector3<Type> operator+(const Vector3<Type>& aVector) const	{return Vector3<Type>(x + aVector.x, y + aVector.y, z + aVector.z);}
    Vector3<Type> operator-(const Vector3<Type>& aVector) const	{return Vector3<Type>(x - aVector.x, y - aVector.y, z - aVector.z);}

    Vector3<Type> operator*(const Vector3<Type>& aVector) const	{return Vector3<Type>(x * aVector.x, y * aVector.y, z * aVector.z);}

    Vector3<Type>  operator*(Type aValue) const							{return Vector3<Type>(x * aValue, y * aValue,  z * aValue);}
    Vector3<Type>  operator/(Type aValue) const							{return Vector3<Type>(x / aValue, y / aValue,  z / aValue);}

    //COMPARISON
	const bool operator == (const Vector3<Type>& aVector) const				{return (x == aVector.x && y == aVector.y && z == aVector.z);}
    const bool operator != (const Vector3<Type>& aVector) const				{return !(*this == aVector);}
	const bool operator >= (const Vector3<Type>& aVector) const				{return x >= aVector.x && y >= aVector.y && z >= aVector.z;}
	const bool operator < (const Vector3<Type>& aVector) const				{return !(*this >= aVector);}

	//NEGATION
	Vector3<Type> operator-() const										{return Vector3<Type>(-x, -y, -z);}

    //MATH
	Type length() const														{return (Type)(sqrt(x * x + y * y + z * z));}
    Type length2() const													{return (x * x + y * y + z * z);}
 	Vector3<Type>& scale(Type aVector)								{Normalize(); operator *= (aVector); return *this;}
    Vector3<Type>& normalize()											{operator /= (length()); return *this;}
    //Vector3<Type>& normalizeSafe()										{(length2() > 0.0) ? Normalize() : x = y = z = 0; return *this;} // CHANGE LATER!!! this gives compiler error, while below line works
    Vector3<Type>& normalizeSafe()										{if(length2() > 0.0) normalize(); else x = y = z = 0; return *this;}
	Vector3<Type> unit() const											{return Vector3<Type>(*this).Normalize();}
	Vector3<Type> unitSafe() const										{return Vector3<Type>(*this).normalizeSafe();}

	Type psi() const														{return atan2(x, z);}
	Type theta() const														{return atan2(y, sqrt(x*x +z*z));}

	Vector3<Type> cross(const Vector3<Type>& aVector) const
	{ 
		Vector3<Type> r;

		r.x = y * aVector.z - z * aVector.y; 
		r.y = z * aVector.x - x * aVector.z;
		r.z = x * aVector.y - y * aVector.x; 

		return r;
	}	

	Type dot(const Vector3<Type>& aVector) const						{return aVector.x * x + aVector.y * y + aVector.z * z;}

	//Rotate the vector around x
	Vector3<Type>& rotateAroundX(float anAngle)			
	{ 
		if(anAngle != 0.f)
		{
			double sinVal = sin(anAngle);
			double cosVal = cos(anAngle);
			Type temp;

			temp = (Type) (y * cosVal + z * sinVal);
			z = (Type) (z * cosVal - y * sinVal);
			y = temp;
		}

		return *this;
	}

	//Rotate the vector around y
	Vector3<Type>& rotateAroundY(float anAngle)			
	{ 
		if(anAngle != 0.f)
		{
			double sinVal = sin(anAngle);
			double cosVal = cos(anAngle);
			Type temp;

			temp = (Type) (x * cosVal + z * sinVal);
			z = (Type) (z * cosVal - x * sinVal);
			x = temp;
		}

		return *this;
	}

	//Rotate the vector around z
	Vector3<Type>& rotateAroundZ(float anAngle)			
	{ 
		if(anAngle != 0.f)
		{
			double sinVal = sin(anAngle);
			double cosVal = cos(anAngle);
			Type temp;

			temp = (Type) (x * cosVal + y * sinVal);
			y = (Type) (y * cosVal - x * sinVal);
			x = temp;
		}

		return *this;
	}

	const Type operator[](const unsigned char anIndex) const
	{
		switch(anIndex)
		{
		case 0:		return x;			break;
		case 1:		return y;			break;
		case 2:		return z;			break;
		default:	return Type(0);		break;
		}
	}

	Type& operator[](const unsigned char anIndex)
	{
		static Type badIndex(0);

		switch(anIndex)
		{
		case 0:		return x;			break;
		case 1:		return y;			break;
		case 2:		return z;			break;
		default:	return badIndex;	break;
		}
	}

	//MEMBERS
	Type x, y, z;
};



//4d vector
template <class Type>
class Vector4 
{
public:

	//CONSTRUCTORS
	Vector4()															{}
    Vector4(Type aX, Type aY, Type aZ, Type aW)				{x = aX; y = aY; z = aZ; w = aW;}

	template <class AnotherType> 
	Vector4(const Vector4<AnotherType>& aVector)					{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; w = (Type)aVector.w;}
    
//	template <class AnotherType> 
//	Vector4(const Vector3<AnotherType>& aVector)					{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; w = (Type)1;}

	//DESTRUCTOR
	~Vector4(){};

	//ASSIGNMENT
	void set(Type aX, Type aY, Type aZ, Type aW)							{x = aX; y = aY; z = aZ; w = aW;}

//	template <class AnotherType> 
//    Vector4<Type>& operator=(const Vector3<AnotherType>& aVector) 	{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; w =Type(1); return *this;}

	template <class AnotherType> 
    Vector4<Type>& operator=(const Vector4<AnotherType>& aVector) 	{x = (Type)aVector.x; y = (Type)aVector.y; z = (Type)aVector.z; w = (Type)aVector.w; return *this;}

    //MODIFYING OPERATORS
	Vector4<Type>& operator+=(const Vector4<Type>& aVector)		{x += aVector.x; y += aVector.y; z += aVector.z; w += aVector.w; return *this;}
    Vector4<Type>& operator-=(const Vector4<Type>& aVector)		{x -= aVector.x; y += aVector.y; z -= aVector.z; w -= aVector.w; return *this;}
    
    Vector4<Type>& operator*=(Type aVector)							{x *= aVector; y *= aVector; z *= aVector; w *= aVector; return *this;}
    Vector4<Type>& operator/=(Type aVector)							{x /= aVector; y /= aVector; z /= aVector; w /= aVector; return *this;}

    //CREATIVE OPERATORS
    Vector4<Type> operator+(const Vector4<Type>& aVector) const	{return Vector4<Type>(x + aVector.x, y + aVector.y, z + aVector.z, w + aVector.w);}
    Vector4<Type> operator-(const Vector4<Type>& aVector) const	{return Vector4<Type>(x - aVector.x, y - aVector.y, z - aVector.z, w - aVector.w);}

    Vector4<Type>  operator*(Type aVector) const						{return Vector4<Type>(x * aVector, y * aVector, z * aVector, w * aVector);}
    Vector4<Type>  operator/(Type aVector) const						{return Vector4<Type>(x / aVector, y / aVector, z / aVector, w / aVector);}

    //COMPARISON
	const bool operator==(const Vector4<Type>& aVector) const				{return (x == aVector.x && y == aVector.y && z == aVector.z && w == aVector.w);}
    const bool operator!=(const Vector4<Type>& aVector) const				{return !(*this == aVector);}

	//NEGATION
	Vector4<Type> operator-() const										{return Vector4<Type>(-x, -y, -z, -w);}
	
	//MATH
	Type length() const														{return (Type) sqrt(x * x + y * y + z * z + w * w);}
	Type length2() const													{return (x * x + y * y + z * z + w * w);}
	Vector4<Type>& scale(Type aVector)								{normalize(); operator *= (aVector); return *this;}
    Vector4<Type>& normalize()											{operator /= (length()); return *this;}
    Vector4<Type>& normalizeSafe()										{if(length2() > 0) normalize(); return *this;}
	Vector4<Type> unit() const											{return Vector4<Type>(*this).normalize();}
	Vector4<Type> unitSafe() const										{return Vector4<Type>(*this).normalizeSafe();}

	//MEMBERS
	union	{Type x;	Type left;};
	union	{Type y;	Type top;};
	union	{Type z;	Type right;};
	union	{Type w;	Type bottom;};
};



//TYPEDEFS
//vector2
typedef Vector2<int> Vector2i;
typedef Vector2<double> Vector2d;
typedef Vector2<float> Vector2f;
typedef Vector2<unsigned short> Vector2us;
typedef Vector2<unsigned char> Vector2uc;
typedef Vector2<unsigned int> Vector2ui;

//vector3
typedef Vector3<int> Vector3i;
typedef Vector3<double> Vector3d;
typedef Vector3<float> Vector3f;
typedef Vector3<unsigned char> Vector3uc;
typedef Vector3<short> Vector3s;
typedef Vector3<unsigned short> Vector3us;
typedef Vector3<unsigned int> Vector3ui;

//vector4
typedef Vector4<int> Vector4i;
typedef Vector4<double> Vector4d;
typedef Vector4<float> Vector4f;
typedef Vector4<unsigned char> Vector4uc;

}
