#include "stdafx.h"
#include "view.h"

#include "canvas.h"
#include "style.h"
#include "idraw.h"

namespace brixx
{

#define WINDOW_CLASS L"Brixx"
static View* locView(NULL);
static IDraw* locDraw(NULL);
static bool locRet(true);
static ::ULONG_PTR locToken(0);

//local methods
static ::LRESULT CALLBACK wndProc(::HWND hWnd, ::UINT message, ::WPARAM wParam, ::LPARAM lParam)
{
	switch(message)
	{
	case WM_PAINT:
		{
			locView->doPaint(hWnd);
			return 0;
		}
		break;

	case WM_DESTROY:
		{
			::PostQuitMessage(0);
			return 0;
		}
		break;

	case WM_SETCURSOR:
		{
			::SetCursor(::LoadCursor(NULL, IDC_ARROW));
			//::SetCursor(NULL);
			return 0;
		}
		break;

	default:
		{
			return ::DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	}
}

View::View(const unsigned int aBackgroundColor, const int aButtonHeight)
:myBackgroundColor(aBackgroundColor)
,myState(aButtonHeight)
{
	locView = this;
}

void View::init(::HINSTANCE hInstance,
				::INT iCmdShow,
				const WCHAR* anIconFile,
				const int aWidth,
				const int aHeight,
				const ::WCHAR* aTitle,
				const int aBrush,
				const ::DWORD aWindowStyle,
				const bool aChroma,
				const ::COLORREF aChromaColor)
{
	::WNDCLASS c;
	Gdiplus::GdiplusStartupInput i;

	// Initialize GDI+.
	Gdiplus::GdiplusStartup(&locToken, &i, NULL);

	//window
	c.style          = CS_HREDRAW | CS_VREDRAW;
	c.lpfnWndProc    = wndProc;
	c.cbClsExtra     = 0;
	c.cbWndExtra     = 0;
	c.hInstance      = hInstance;
	if(anIconFile)
		c.hIcon = (::HICON)::LoadImage(NULL, anIconFile, IMAGE_ICON, 0, 0, LR_LOADFROMFILE);
	else
		c.hIcon = ::LoadIcon(NULL, IDI_APPLICATION);
	c.hCursor        = ::LoadCursor(NULL, IDC_ARROW);
	c.hbrBackground  = (HBRUSH)::GetStockObject(aBrush);
	c.lpszMenuName   = NULL;
	c.lpszClassName  = WINDOW_CLASS;

	::RegisterClass(&c);

	myState.myWindow = ::CreateWindow(
									WINDOW_CLASS,   // window class name
									aTitle,  // window caption
									aWindowStyle,      // window style
									(::GetSystemMetrics(SM_CXSCREEN) - aWidth) / 2,            // initial x position
									(::GetSystemMetrics(SM_CYSCREEN) - aHeight) / 2,            // initial y position
									aWidth,            // initial x size
									aHeight,            // initial y size
									NULL,                     // parent window handle
									NULL,                     // window menu handle
									hInstance,                // program instance handle
									NULL);                    // creation parameters

	if(aChroma)
	{
		::SetWindowLong(myState.myWindow, GWL_EXSTYLE, ::GetWindowLong(myState.myWindow, GWL_EXSTYLE) | WS_EX_LAYERED);
		::SetLayeredWindowAttributes(myState.myWindow, aChromaColor, 0, LWA_COLORKEY);
	}

	::ShowWindow(myState.myWindow, iCmdShow);
	::UpdateWindow(myState.myWindow);
}

void View::cleanup()
{
	Gdiplus::GdiplusShutdown(locToken);
}

const bool View::draw(IDraw& aDraw)
{
	if(isIconic())
	{
		return true;
	}
	else
	{
		locDraw = &aDraw;
		::InvalidateRect(myState.myWindow, NULL, false);
		return locRet;
	}
}

const bool View::isIconic() const
{
	return ::IsIconic(myState.myWindow) == TRUE;
}

//private
void View::doPaint(const ::HWND aWindow)
{
	::HDC hdc;
	::PAINTSTRUCT ps;

	hdc = ::BeginPaint(aWindow, &ps);
	Gdiplus::Graphics g(hdc);
	if(locDraw && locView)
	{
		//canvas size of window
		::RECT rect;
		::GetClientRect(aWindow, &rect);
		Canvas c(locView->myState, rect.right, rect.bottom, locView->myBackgroundColor);

		//callback
		locRet = locDraw->draw(c);

		//present
		c.render(g);

		//sample new input
		locView->myState.update();
	}
	::EndPaint(aWindow, &ps);
}

}
