#include "stdafx.h"

#include "model.h"
#include "view.h"
#include "assets.h"
#include "metacontroller.h"
#include "time.h"
#include "timer.h"

INT WINAPI WinMain(::HINSTANCE hInstance, ::HINSTANCE, ::PSTR aCommandLine, INT iCmdShow)
{
#ifdef _DEBUG
	//turn on memory leak checks
	int flags(::_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG));
	flags |= /*_CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_CHECK_CRT_DF |*/ _CRTDBG_DELAY_FREE_MEM_DF | _CRTDBG_LEAK_CHECK_DF;
	::_CrtSetDbgFlag(flags);
#endif

	//view
	enum
	{
		BUTTON_HEIGHT = 16,
		MENU_WIDTH = 64,
		WIDTH = brixx::Model::PLAYFIELD_WIDTH + MENU_WIDTH,
		HEIGHT = brixx::Model::PLAYFIELD_HEIGHT + brixx::Model::PADDLE_HEIGHT,
	};
	brixx::View v(0xffB79A88, BUTTON_HEIGHT);
	v.init(hInstance, iCmdShow, NULL, WIDTH, HEIGHT, L"Brixx", BLACK_BRUSH, WS_POPUPWINDOW, false, 0);

	::MSG msg;
	//this block is here so that the assets destruct before the view is cleaned up
	{
		//this struct is simply for construction order issues (model and controller ref each other)
		struct App
		{
			App()
			:myModel(myController.myPlay)
			,myController(myAssets, myModel, myModel)
			{
			}

			//members
			brixx::Model myModel;
			brixx::Assets myAssets;
			brixx::MetaController myController;
		} app;

		if(!app.myAssets.init())
			return -1;

		//main loop
		brixx::Timer t(brixx::Model::MILLISECONDS_PER_TICK);
		while(1)
		{
			if(::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
			{
				if(!::GetMessage(&msg, NULL, 0, 0 ))
					break;

				::TranslateMessage(&msg); 
				::DispatchMessage(&msg);
			}
			else
			{
				brixx::time::update();

				t.update();
				if(t.ready() && !v.isIconic())
				{
					if(app.myController.playing())
						app.myModel.update();

					if(!v.draw(app.myController))
						::PostQuitMessage(-1);
				}
			}
		}
	}

	//cleanup
	v.cleanup();

	return (int)msg.wParam;
}
